/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.plugin.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.tinygroup.commons.version.VersionCompareUtil;
import org.tinygroup.i18n.I18nMessageFactory;
import org.tinygroup.i18n.I18nMessages;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.parser.filter.NameFilter;
import org.tinygroup.plugin.Plugin;
import org.tinygroup.plugin.PluginManager;
import org.tinygroup.plugin.config.PluginConfiguration;
import org.tinygroup.plugin.config.PluginInfo;
import org.tinygroup.plugin.config.PluginService;
import org.tinygroup.plugin.exception.PluginException;
import org.tinygroup.plugin.util.PluginUtil;
import org.tinygroup.plugin.util.PluginXmlConfigNode;
import org.tinygroup.springutil.SpringUtil;
import org.tinygroup.xmlparser.node.XmlNode;

/**
 * 插件基类，大多数的情况下，可以不必自己写插件实现类
 * 
 * @author luoguo
 * 
 */
public abstract class BasePlugin implements Plugin {
	private PluginManager pluginManager;
	private PluginInfo pluginInfo;
	private I18nMessages i18nMessages = I18nMessageFactory.getI18nMessages();
	// 存放服务和服务对应的最高版本信息
	private Map<String, PluginService> idInfoMap = new HashMap<String, PluginService>();
	// 存放服务和所有服务的信息
	private Map<String, PluginService> idVersionInfoMap = new HashMap<String, PluginService>();
	// 存放服务实例
	private Map<PluginService, Object> serviceInstanceMap = new HashMap<PluginService, Object>();
	/**
	 * application.xml中的配置信息，每条记录对应一个pluginService
	 */
	private List<XmlNode> configNodeList;
	private Logger logger = LoggerFactory.getLogger(BasePlugin.class);

	public void init() {

		logger.logMessage(LogLevel.DEBUG, "初始化插件[id:{0},version:{1}]的服务",
				pluginInfo.getId(), pluginInfo.getVersion());

		List<PluginService> list = this.pluginInfo.getPluginServices();
		for (PluginService service : list) {
			logger.logMessage(LogLevel.DEBUG, "初始化插件服务[id:{0},version:{1}]",
					service.getId(), service.getVersion());
			String id = service.getId();
			if (idInfoMap.containsKey(id)) {
				int compare = VersionCompareUtil.compareVersion(
						idInfoMap.get(id).getVersion(), service.getVersion());
				if (compare == -1) {
					idInfoMap.put(id, service);
					logger.logMessage(LogLevel.DEBUG,
							"插件服务[id:{0},version:{1}]是当前最高版本", service.getId(),
							service.getVersion());
				}
			} else {
				idInfoMap.put(id, service);
				logger.logMessage(LogLevel.DEBUG,
						"插件服务[id:{0},version:{1}]是当前最高版本", service.getId(),
						service.getVersion());
			}
			initServiceInstance(service);
			idVersionInfoMap.put(getKey(service), service);
			logger.logMessage(LogLevel.DEBUG, "初始化插件服务[id:{0},version:{1}]完成",
					service.getId(), service.getVersion());
		}

		if (pluginManager == null) {
			pluginManager = SpringUtil.getBean("pluginManager");
		}
		XmlNode config = pluginManager.getConifg(getPluginInfo());
		if (config != null) {
			NameFilter<XmlNode> nameFilter = new NameFilter<XmlNode>(config);
			configNodeList = nameFilter
					.findNodeList(PluginXmlConfigNode.SERVICE_NODE);
		}

		logger.logMessage(LogLevel.DEBUG, "初始化插件[id:{0},version:{1}]的服务完成",
				pluginInfo.getId(), pluginInfo.getVersion());

	}

	protected Map<PluginService, Object> getAllServiceMap() {
		return serviceInstanceMap;
	}

	/**
	 * 根据插件服务id和版本，获取对应的配置信息
	 * 
	 * @param serviceId
	 *            插件服务Id
	 * @param serviceVersion
	 *            插件服务版本
	 * @return
	 */
	protected XmlNode findServiceConfig(String serviceId, String serviceVersion) {
		if (configNodeList != null) {
			if (serviceId == null || "".equals(serviceId)
					|| serviceVersion == null || "".equals(serviceVersion)) {
				return null;
			}
			for (XmlNode node : configNodeList) {
				if (serviceId.equals(node
						.getAttribute(PluginXmlConfigNode.SERVICE_ID))
						&& serviceVersion
								.equals(node
										.getAttribute(PluginXmlConfigNode.SERVICE_VERSION))) {
					return node;
				}
			}
		}
		return null;
	}

	protected abstract ClassLoader getThisClassLoader();

	/**
	 * 子类可以通过更新写方法来变化实例的创建方式
	 * 
	 * @param service
	 */
	protected void initServiceInstance(PluginService service) {
		try {

			String className = service.getClassName();
			Class<?> clazz = getThisClassLoader().loadClass(className);
			Object bean = null;
			if (service.getInstanceGetMode() == null
					|| service.getInstanceGetMode().equalsIgnoreCase("new")) {
				bean = clazz.newInstance();
			} else {
				bean = SpringUtil.getBean(service.getId());
				if (bean == null) {
					bean = SpringUtil.getBean(clazz);
				}
			}
			if (bean != null) {
				serviceInstanceMap.put(service, bean);
			} else {
				logger.logMessage(LogLevel.ERROR,
						"服务[id:{0}，version:{1}]的实例获取失败，请检查配置！",
						service.getId(), service.getVersion());
			}
		} catch (Exception e) {
			logger.errorMessage("初始化插件服务[id:{0},version:{1}]出错,", e,
					service.getId(), service.getVersion());
			throw new PluginException(i18nMessages.getMessage(
					"plugin.serviceClassNotFound", service.getId(),
					service.getVersion()), e);
		}
	}

	protected String getKey(PluginService service) {
		return getKey(service.getId(), service.getVersion());
	}

	protected String getKey(String id, String version) {
		return PluginUtil.getKey(id, version);
	}

	public void destroy() {
		idInfoMap.clear();
		idVersionInfoMap.clear();
		serviceInstanceMap.clear();
		if (configNodeList != null) {
			configNodeList.clear();
		}
	}

	public <T> T getService(String id, String version) {
		if (id == null || "".equals(id)) {
			throw new PluginException(
					i18nMessages.getMessage("plugin.serviceIdIsNull"));
		}
		if (version == null) {
			return getService(id);
		}
		PluginService service = idVersionInfoMap.get(getKey(id, version));
		return getService(service);

	}

	public <T> T getService(String id) {
		PluginService service = idInfoMap.get(id);
		return getService(service);
	}

	protected <T> T getService(PluginService service) {
		if (service == null) {
			return null;
		}
		Object object = serviceInstanceMap.get(service);
		if (object == null) {
			return null;
		}
		return (T) object;
	}

	public void setPluginManager(PluginManager pluginManager) {
		this.pluginManager = pluginManager;
	}

	public PluginManager getPluginManager() {
		return pluginManager;
	}

	public <T> void setPluginInfo(PluginInfo pluginInfo) {
		this.pluginInfo = pluginInfo;
	}

	public void start() {

	}

	public void pause() {

	}

	public void stop() {

	}

	public <T> T getService(Class<T> clazz, String version) {
		logger.logMessage(LogLevel.DEBUG, "通过Class与version获取插件服务");
		if (version == null || "".equals(version)) {
			logger.logMessage(LogLevel.DEBUG,
					"通过Class与version获取插件服务,传入的version为空");
			return null;
		}
		Set<Entry<PluginService, Object>> set = this.serviceInstanceMap
				.entrySet();
		Iterator<Entry<PluginService, Object>> iterator = set.iterator();

		while (iterator.hasNext()) {
			Entry<PluginService, Object> entry = iterator.next();
			Object o = entry.getValue();
			if (o.getClass().isAssignableFrom(clazz)) {
				PluginService service = entry.getKey();
				if (version.equals(service.getVersion())) {
					logger.logMessage(LogLevel.DEBUG,
							"通过Class与version获取插件服务,返回服务[id{0},version:{1}]",
							service.getId(), service.getVersion());
					return (T) o;
				}
			}
		}
		logger.logMessage(LogLevel.DEBUG, "通过Class与version获取插件服务,未查找到合适的服务");
		return null;
	}

	public <T> T getService(Class<T> clazz) {
		logger.logMessage(LogLevel.DEBUG, "通过Class获取插件服务");
		Set<Entry<PluginService, Object>> set = this.serviceInstanceMap
				.entrySet();
		Iterator<Entry<PluginService, Object>> iterator = set.iterator();
		PluginService service = null;
		while (iterator.hasNext()) {
			Entry<PluginService, Object> entry = iterator.next();
			Object o = entry.getValue();
			if (o.getClass().isAssignableFrom(clazz)) {
				service = entry.getKey();
				break;
			}
		}
		if (service == null) {
			logger.logMessage(LogLevel.DEBUG, "通过Class获取插件服务,未查找到合适的服务");
			return null;
		}
		// 获取当前id对应service的最高版本
		service = idInfoMap.get(service.getId());
		logger.logMessage(LogLevel.DEBUG,
				"通过Class获取插件服务,返回服务[id{0},version:{1}]", service.getId(),
				service.getVersion());
		return (T) serviceInstanceMap.get(service);
	}

	public <T> void setService(T service, Class<T> clazz) {

	}

	public <T> void setConfig(PluginConfiguration pluginConfiguration) {

	}

	public Map<String, PluginService> getIdInfoMap() {
		return idInfoMap;
	}

	public Map<String, PluginService> getIdVersionInfoMap() {
		return idVersionInfoMap;
	}

	public Map<PluginService, Object> getServiceInstanceMap() {
		return serviceInstanceMap;
	}

	public PluginInfo getPluginInfo() {
		return pluginInfo;
	}

}
