/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.plugin.impl;

import static org.tinygroup.logger.LogLevel.INFO;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tinygroup.context.Context;
import org.tinygroup.context.impl.ContextImpl;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.parser.filter.NameFilter;
import org.tinygroup.plugin.Plugin;
import org.tinygroup.plugin.PluginManager;
import org.tinygroup.plugin.config.PluginConfiguration;
import org.tinygroup.plugin.config.PluginDependency;
import org.tinygroup.plugin.config.PluginInfo;
import org.tinygroup.plugin.exception.PluginException;
import org.tinygroup.plugin.util.PluginUtil;
import org.tinygroup.plugin.util.PluginXmlConfigNode;
import org.tinygroup.xmlparser.node.XmlNode;

/**
 * 插件管理器
 * 
 * @author luoguo
 * 
 */
public class PluginManagerImpl implements PluginManager {

	/**
	 * application.xml中plugin配置信息节点名
	 */
	private static final String PLUGINCONFIGNODE = PluginXmlConfigNode.PLUGIN_CONFIGS;
	/**
	 * application.xml中plugin配置信息节点xml
	 */
	private XmlNode config;
	/**
	 * application.xml中每个插件(id+版本)的配置信息(从config中解析出来的)
	 */
	private Map<String, XmlNode> pluginConfigs = new HashMap<String, XmlNode>();

	private Logger logger = LoggerFactory.getLogger(PluginManagerImpl.class);
	/**
	 * key=id+"-"+version
	 */
	private final Map<String, PluginInfo> idVersionMap = new HashMap<String, PluginInfo>();
	/**
	 * key=id
	 */
	private final Map<String, PluginInfo> idMap = new HashMap<String, PluginInfo>();
	/**
	 * 比较器
	 */
	private PluginInfoCompare pluginInfoCompare = new PluginInfoCompareImpl();
	private final Map<PluginInfo, Plugin> pluginMap = new HashMap<PluginInfo, Plugin>();
	private final Map<PluginInfo, Integer> pluginStatus = new HashMap<PluginInfo, Integer>();
	private final List<PluginInfo> pluginInfoList = new ArrayList<PluginInfo>();
	private Context context = new ContextImpl();


	public PluginInfoCompare getPluginInfoCompare() {
		return pluginInfoCompare;
	}

	public void setPluginInfoCompare(PluginInfoCompare pluginInfoCompare) {
		this.pluginInfoCompare = pluginInfoCompare;
	}

	public void add(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始加载插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		if (idVersionMap.containsKey(getKey(pluginInfo))) {
			logger.logMessage(INFO,
					"插件[id:{0}, version:{1}, type:{2}]已存在，无需加载",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType());
			return;
		}

		pluginInfoList.add(pluginInfo);
		idVersionMap.put(getKey(pluginInfo), pluginInfo);

		try {
			logger.logMessage(INFO, "开始创建插件[id:{0}, version:{1}, type:{2}]实例",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType());

			logger.logMessage(INFO, "插件[id:{0}, version:{1}, type:{2}]实例已创建完成",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType());
			Plugin plugin = null;
			logger.logMessage(LogLevel.INFO,
					"插件[id:{0}, version:{1}, type:{2}]对应的loader不存在",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType());
			logger.logMessage(LogLevel.INFO, "采用系统Loader实例化");
			plugin = (Plugin) Class.forName(pluginInfo.getType()).newInstance();
			plugin.setPluginManager(this);
			plugin.setPluginInfo(pluginInfo);
			pluginMap.put(pluginInfo, plugin);
			PluginConfiguration pluginConfiguration = pluginInfo
					.getPluginConfiguration();
			if (pluginConfiguration != null
					&& pluginConfiguration.getConfigurations() != null) {
				plugin.setConfig(pluginConfiguration);
			}
			PluginInfo info = idMap.get(pluginInfo.getId());
			if (info == null) {
				idMap.put(pluginInfo.getId(), pluginInfo);
			} else if (pluginInfoCompare.compare(pluginInfo, info) > 0) {
				idMap.put(pluginInfo.getId(), pluginInfo);
			}
			logger.log(INFO, "plugin.add", pluginInfo.getId(),
					pluginInfo.getVersion());
		} catch (Exception e) {
			logger.errorMessage("加载插件[id:{0}, version:{1}, type:{2}]时出错", e,
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType());
		}
	}


	private String getKey(PluginInfo pluginInfo) {
		return getKey(pluginInfo.getId(), pluginInfo.getVersion());
	}

	private String getKey(String id, String version) {
		return PluginUtil.getKey(id, version);
	}

	public void add(PluginInfo[] pluginInfos) {
		for (PluginInfo pluginInfo : pluginInfos) {
			add(pluginInfo);
		}
	}

	public void add(List<PluginInfo> pluginInfolist) {
		for (PluginInfo pluginInfo : pluginInfolist) {
			add(pluginInfo);
		}
	}

	public int status(PluginInfo pluginInfo) {
		int status = PluginManager.STATUS_UNINITIALIZED;
		synchronized (pluginStatus) {
			Integer s = pluginStatus.get(pluginInfo);
			if (s != null) {
				status = s;
			}
		}
		return status;
	}

	public void init(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始初始化插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		int status = status(pluginInfo);// 获取插件初始状态
		// 判断是否是未初始化
		if (status != PluginManager.STATUS_UNINITIALIZED) {
			logger.logMessage(INFO, "插件[id:{0}, version:{1}]状态是[{2}],退出初始化",
					pluginInfo.getId(), pluginInfo.getVersion(),
					getStatusDescription(status));
			return;
		}

		// 初始化插件实例
		try {
			setStatus(pluginInfo, PluginManager.STATUS_INITING);
			pluginMap.get(pluginInfo).init();
			setStatus(pluginInfo, PluginManager.STATUS_INITED);
		} catch (PluginException e) {
			setStatus(pluginInfo, status);
			logger.error("插件[id:{0}, version:{1}, type:{2}]初始化时出错,恢复状态为[{3}]",
					e, pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), getStatusDescription(status));
			throw e;
		}
		logger.log(INFO, "plugin.init", pluginInfo.getId(),
				pluginInfo.getVersion());
	}

	private void setStatus(PluginInfo pluginInfo, int status) {
		synchronized (pluginStatus) {
			pluginStatus.put(pluginInfo, status);
		}
	}

	public void start(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始启动插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		int status = status(pluginInfo);
		// 如果是已start则返回
		if (status == PluginManager.STATUS_READY) {

			logger.logMessage(LogLevel.INFO,
					"插件[id:{0}, version:{1}, type:{2}]状态是[{3}]，退出启动",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), getStatusDescription(status));

			return;
		}

		// 如果未初始化,则初始化
		if (status == PluginManager.STATUS_UNINITIALIZED) {
			init(pluginInfo);
			status = status(pluginInfo);
		}
		// 如果未组装,则组装
		if (status == PluginManager.STATUS_INITED) {
			assemble(pluginInfo);
			status = status(pluginInfo);
		}
		// 如果是组装OK或暂停状态
		if (status == PluginManager.STATUS_ASSEMBLEED
				|| status == PluginManager.STATUS_PAUSED) {
			try {

				logger.logMessage(LogLevel.INFO,
						"插件[id:{0}, version:{1}, type:{2}]状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));

				Plugin plugin = pluginMap.get(pluginInfo);
				setStatus(pluginInfo, PluginManager.STATUS_STARTING);
				// 先启动依赖的项
				startDependency(pluginInfo);
				plugin.start();
				setStatus(pluginInfo, PluginManager.STATUS_READY);
			} catch (PluginException e) {
				setStatus(pluginInfo, status);
				logger.logMessage(LogLevel.ERROR,
						"插件[id:{0}, version:{1}, type:{2}]启动时出错，恢复状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));
				throw e;
			}
		}
		logger.log(INFO, "plugin.start", pluginInfo.getId(),
				pluginInfo.getVersion());
	}

	private void startDependency(PluginInfo pluginInfo) {
		logger.logMessage(LogLevel.INFO,
				"启动插件[id:{0}, version:{1}, type:{2}]依赖的项", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());
		if (pluginInfo.getPluginDependencies() != null) {
			for (PluginDependency pluginDependency : pluginInfo
					.getPluginDependencies()) {

				logger.logMessage(LogLevel.INFO,
						"启动依赖的项[plugin-id:{0},plugin-version:{1}]",
						pluginDependency.getPluginId(),
						pluginDependency.getPluginVersion());

				PluginInfo dependPlugin = getDependencyPluginInfo(pluginDependency);
				start(dependPlugin);

				logger.logMessage(LogLevel.INFO,
						"依赖的项[plugin-id:{0},plugin-version:{1}]启动完成",
						pluginDependency.getPluginId(),
						pluginDependency.getPluginVersion());
			}
		}
		logger.logMessage(LogLevel.INFO,
				"启动插件[id:{0}, version:{1}, type:{2}]依赖的项完成",
				pluginInfo.getId(), pluginInfo.getVersion(),
				pluginInfo.getType());
	}

	public void stop(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始停止插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		int status = status(pluginInfo);
		// 如果状态已是已装配，则直接返回
		if (status == PluginManager.STATUS_ASSEMBLEED) {
			logger.logMessage(LogLevel.INFO,
					"插件[id:{0}, version:{1}, type:{2}]状态为[{3}]，退出停止",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), getStatusDescription(status));
			return;
		}

		// 如果是初始化OK或暂停状态
		if (status == PluginManager.STATUS_READY
				|| status == PluginManager.STATUS_PAUSED) {
			try {
				logger.logMessage(LogLevel.INFO,
						"插件[id:{0}, version:{1}, type:{2}]状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));

				Plugin plugin = pluginMap.get(pluginInfo);
				setStatus(pluginInfo, PluginManager.STATUS_STOPING);
				// 先要停止依赖当前插件的项
				stopDependencyBy(pluginInfo);
				plugin.stop();
				setStatus(pluginInfo, PluginManager.STATUS_ASSEMBLEED);
			} catch (PluginException e) {
				setStatus(pluginInfo, status);
				logger.logMessage(LogLevel.ERROR,
						"插件[id:{0}, version:{1}, type:{2}]停止时出错，恢复状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));
				throw e;
			}
		}
		logger.log(INFO, "plugin.stop", pluginInfo.getId(),
				pluginInfo.getVersion());
	}

	private void stopDependencyBy(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始停止依赖插件[id:{0}, version:{1}, type:{2}]的项",
				pluginInfo.getId(), pluginInfo.getVersion(),
				pluginInfo.getType());
		for (PluginInfo info : pluginInfoList) {
			if (info.getPluginDependencies() != null) {
				for (PluginDependency pluginDependency : info
						.getPluginDependencies()) {
					PluginInfo dependency = getDependencyPluginInfo(pluginDependency);
					if (dependency.equals(pluginInfo)) {
						logger.logMessage(LogLevel.INFO,
								"开始停止依赖项[id:{0},version:{1}]",
								dependency.getId(), dependency.getVersion());
						stop(info);
						logger.logMessage(LogLevel.INFO,
								"依赖项[id:{0},version:{1}]停止完成",
								dependency.getId(), dependency.getVersion());
					}
				}
			}
		}
		logger.logMessage(LogLevel.INFO,
				"停止依赖插件[id:{0}, version:{1}, type:{2}]的项完成",
				pluginInfo.getId(), pluginInfo.getVersion(),
				pluginInfo.getType());
	}

	private PluginInfo getDependencyPluginInfo(PluginDependency pluginDependency) {
		PluginInfo dependency = null;
		if (pluginDependency.getPluginVersion() == null) {
			dependency = getPluginInfo(pluginDependency.getPluginId());
		} else {
			dependency = getPluginInfo(pluginDependency.getPluginId(),
					pluginDependency.getPluginVersion());
		}
		if (dependency == null) {
			throw new PluginException(String.format(
					"插件[plugin-id:%s,plugin-version:%s]不存在",
					pluginDependency.getPluginId(),
					pluginDependency.getPluginVersion()));
		}
		return dependency;
	}

	public void pause(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始暂停插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		int status = status(pluginInfo);
		// 如果是初始化OK或暂停状态
		if (status == PluginManager.STATUS_READY) {
			try {
				logger.logMessage(LogLevel.INFO,
						"插件[id:{0}, version:{1}, type:{2}]状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));

				Plugin plugin = pluginMap.get(pluginInfo);
				setStatus(pluginInfo, PluginManager.STATUS_PAUSING);
				// 先要暂停依赖的项
				pauseDependencyBy(pluginInfo);
				plugin.pause();
				setStatus(pluginInfo, PluginManager.STATUS_PAUSED);
			} catch (PluginException e) {
				setStatus(pluginInfo, status);
				logger.logMessage(LogLevel.ERROR,
						"插件[id:{0}, version:{1}, type:{2}]暂停时出错，恢复状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));
				throw e;
			}
		} else {
			logger.logMessage(LogLevel.INFO,
					"插件[id:{0}, version:{1}, type:{2}]状态为[{3}]，退出暂停",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), getStatusDescription(status));
			return;
		}
		logger.log(INFO, "plugin.pause", pluginInfo.getId(),
				pluginInfo.getVersion());
	}

	private void pauseDependencyBy(PluginInfo pluginInfo) {
		logger.logMessage(LogLevel.INFO,
				"暂停依赖插件[id:{0}, version:{1}, type:{2}]的项", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());
		for (PluginInfo info : pluginInfoList) {
			if (info.getPluginDependencies() != null) {
				for (PluginDependency pluginDependency : info
						.getPluginDependencies()) {
					PluginInfo dependency = getDependencyPluginInfo(pluginDependency);
					if (dependency.equals(pluginInfo)) {
						logger.logMessage(LogLevel.INFO,
								"暂停依赖项[id:{0},version{1}]", dependency.getId(),
								dependency.getVersion());
						pause(info);
						logger.logMessage(LogLevel.INFO,
								"依赖项[id:{0},version{1}]暂停完成",
								dependency.getId(), dependency.getVersion());
					}
				}
			}
		}

		logger.logMessage(LogLevel.INFO,
				"暂停依赖插件[id:{0}, version:{1}, type:{2}]的项完成",
				pluginInfo.getId(), pluginInfo.getVersion(),
				pluginInfo.getType());
	}

	public void destroy(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始销毁插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		int status = status(pluginInfo);
		/**
		 * 如果是未初始化，则结束
		 */
		if (status == PluginManager.STATUS_UNINITIALIZED) {
			logger.logMessage(LogLevel.INFO,
					"插件[id:{0}, version:{1}, type:{2}]状态为[{3}]，退出销毁",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), getStatusDescription(status));
			return;
		}
		// 只有是停止状态才可以被销毁
		if (status == PluginManager.STATUS_READY
				|| status == PluginManager.STATUS_PAUSED) {

			stop(pluginInfo);
			status = status(pluginInfo);
		}
		// 如果已经停止,则分解它
		if (status == PluginManager.STATUS_ASSEMBLEED) {
			disassemble(pluginInfo);
			status = status(pluginInfo);
		}

		if (status == PluginManager.STATUS_INITED) {
			try {
				Plugin plugin = pluginMap.get(pluginInfo);
				setStatus(pluginInfo, PluginManager.STATUS_DESTORYING);
				disassembleDependencyBy(pluginInfo);
				plugin.destroy();
				// pluginMap.remove(pluginInfo); 20120831不应该remover，否则插件无法再次被初始化
				setStatus(pluginInfo, PluginManager.STATUS_UNINITIALIZED);
			} catch (PluginException e) {
				setStatus(pluginInfo, status);
				logger.logMessage(LogLevel.ERROR,
						"插件[id:{0}, version:{1}, type:{2}]销毁时出错，恢复状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));
				throw e;
			}
		}

		logger.log(INFO, "plugin.destroy", pluginInfo.getId(),
				pluginInfo.getVersion());
	}

	private void disassembleDependencyBy(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"卸载依赖插件[id:{0}, version:{1}, type:{2}]的项", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		for (PluginInfo info : pluginInfoList) {
			if (!info.equals(pluginInfo)
					&& info.getPluginDependencies() != null) {
				for (PluginDependency pluginDependency : info
						.getPluginDependencies()) {
					PluginInfo dependency = getDependencyPluginInfo(pluginDependency);
					if (dependency.equals(pluginInfo)) {
						logger.logMessage(LogLevel.INFO,
								"卸载依赖项[id:{0}, version:{1}]",
								dependency.getId(), dependency.getVersion());
						disassemble(info);
						logger.logMessage(LogLevel.INFO,
								"卸载依赖项[id:{0}, version:{1}]完成",
								dependency.getId(), dependency.getVersion());
						cleanDependencyService(info, pluginInfo,
								pluginDependency);
					}
				}
			}
		}

		logger.logMessage(LogLevel.INFO,
				"卸载依赖插件[id:{0}, version:{1}, type:{2}]的项完成",
				pluginInfo.getId(), pluginInfo.getVersion(),
				pluginInfo.getType());

	}

	/**
	 * 清空依赖的服务
	 * 
	 * @param pluginInfo
	 *            依赖者
	 * @param dependPluginInfo
	 *            被依赖者
	 * @param pluginDependency
	 *            依赖描述
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void cleanDependencyService(PluginInfo pluginInfo,
			PluginInfo dependPluginInfo, PluginDependency pluginDependency) {

		logger.logMessage(
				LogLevel.INFO,
				"清空插件[id:{0}, version:{1}, type:{2}]依赖的服务[plugin-id:{3},plugin-version:{4},service-id={5},service-version={6}]",
				pluginInfo.getId(), pluginInfo.getVersion(),
				pluginInfo.getType(), pluginDependency.getPluginId(),
				pluginDependency.getPluginVersion(),
				pluginDependency.getServiceId(),
				pluginDependency.getServiceVersion());

		String serviceVersion = pluginDependency.getServiceVersion();
		String serviceId = pluginDependency.getServiceId();
		String serviceType = pluginDependency.getServiceType();
		try {
			if (serviceVersion == null) {
				if (serviceId == null) {
					Class clazz = Class.forName(serviceType);
					pluginMap.get(pluginInfo).setService(null, clazz);

				} else {
					Object object = pluginMap.get(dependPluginInfo).getService(
							serviceId);
					Class clazz = object.getClass();
					pluginMap.get(pluginInfo).setService(null, clazz);
				}
			} else {
				if (serviceId == null) {
					Class clazz = Class.forName(serviceType);
					pluginMap.get(pluginInfo).setService(null, clazz);

				} else {
					Object object = pluginMap.get(dependPluginInfo).getService(
							serviceId, serviceVersion);
					Class clazz = object.getClass();
					pluginMap.get(pluginInfo).setService(null, clazz);
				}
			}
			logger.logMessage(
					LogLevel.INFO,
					"清空插件[id:{0}, version:{1}, type:{2}]依赖的服务[plugin-id:{3},plugin-version:{4},service-id={5},service-version={6}]完成",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), pluginDependency.getPluginId(),
					pluginDependency.getPluginVersion(),
					pluginDependency.getServiceId(),
					pluginDependency.getServiceVersion());
		} catch (Exception e) {
			logger.logMessage(
					LogLevel.INFO,
					"清空插件[id:{0}, version:{1}, type:{2}]依赖的服务[plugin-id:{3},plugin-version:{4},service-id={5},service-version={6}]时出错",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), pluginDependency.getPluginId(),
					pluginDependency.getPluginVersion(),
					pluginDependency.getServiceId(),
					pluginDependency.getServiceVersion());
			throw new PluginException(e.getMessage(), e);
		}
	}

	public void init() {
		logger.logMessage(LogLevel.INFO, "开始初始化所有插件");
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				init(pluginInfo);
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "所有插件已初始化");
	}

	public void start() {
		logger.logMessage(LogLevel.INFO, "开始启动所有插件");
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				start(pluginInfo);
			} catch (PluginException e) {

			}
		}
		logger.logMessage(LogLevel.INFO, "所有插件已启动");
	}

	public void stop() {
		logger.logMessage(LogLevel.INFO, "开始停止所有插件");
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				stop(pluginInfo);
			} catch (PluginException e) {
				processException(e);

			}
		}
		logger.logMessage(LogLevel.INFO, "所有插件已停止");
	}

	public void pause() {
		logger.logMessage(LogLevel.INFO, "开始暂停所有插件");
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				pause(pluginInfo);
			} catch (PluginException e) {
				processException(e);

			}
		}
		logger.logMessage(LogLevel.INFO, "所有插件已暂停");
	}

	public void destroy() {
		logger.logMessage(LogLevel.INFO, "开始销毁所有插件");
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				destroy(pluginInfo);
			} catch (PluginException e) {
				processException(e);

			}
		}
		logger.logMessage(LogLevel.INFO, "所有插件已销毁");
	}

	private void processException(PluginException e) {
		logger.errorMessage(e.getMessage(), e);
	}

	public <T> T getService(PluginInfo pluginInfo, Class<T> clazz) {
		int status = status(pluginInfo);
		if (status == PluginManager.STATUS_READY) {
			return pluginMap.get(pluginInfo).getService(clazz);
		}
		throw getNotReadyException();
	}

	private PluginException getNotReadyException() {
		logger.logMessage(LogLevel.ERROR, "插件状态不是STATUS_READY");
		return new PluginException("插件状态不是STATUS_READY");
	}

	public <T> T getService(PluginInfo pluginInfo, Class<T> clazz,
			String version) {
		int status = status(pluginInfo);
		if (status == PluginManager.STATUS_READY) {
			return pluginMap.get(pluginInfo).getService(clazz, version);
		}
		throw getNotReadyException();

	}

	public PluginInfo getPluginInfo(String id, String version) {
		return idVersionMap.get(getKey(id, version));
	}

	public PluginInfo getPluginInfo(String id) {
		return idMap.get(id);
	}

	public <T> T getService(PluginInfo pluginInfo, String id) {
		int status = status(pluginInfo);
		if (status == PluginManager.STATUS_READY) {
			return  pluginMap.get(pluginInfo).getService(id);
		}
		throw getNotReadyException();
	}

	public <T> T getService(PluginInfo pluginInfo, String id, String version) {
		int status = status(pluginInfo);
		if (status == PluginManager.STATUS_READY) {
			return pluginMap.get(pluginInfo).getService(id, version);
		}
		throw getNotReadyException();
	}

	public void assemble(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始装载插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		int status = status(pluginInfo);
		// 如果是未初始化状态，先初始化之
		if (status == PluginManager.STATUS_UNINITIALIZED) {
			init(pluginInfo);
			status = status(pluginInfo);
		}
		// 如果是已初始化状态
		if (status == PluginManager.STATUS_INITED) {
			try {
				setStatus(pluginInfo, PluginManager.STATUS_ASSEMBLEING);
				assembleDependency(pluginInfo);
				setStatus(pluginInfo, PluginManager.STATUS_ASSEMBLEED);
			} catch (PluginException e) {
				setStatus(pluginInfo, status);
				logger.logMessage(LogLevel.ERROR,
						"插件[id:{0}, version:{1}, type:{2}]装载时出错，恢复状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));
				throw e;
			}
		}

		logger.logMessage(LogLevel.INFO,
				"插件[id:{0}, version:{1}, type:{2}]已装载完成", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());
	}

	private void assembleDependency(PluginInfo pluginInfo) {
		logger.logMessage(LogLevel.INFO,
				"装载插件[id:{0}, version:{1}, type:{2}]依赖的项", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		if (pluginInfo.getPluginDependencies() != null) {
			for (PluginDependency pluginDependency : pluginInfo
					.getPluginDependencies()) {

				logger.logMessage(LogLevel.INFO,
						"装载依赖的项[plugin-id:{0},plugin-version:{1}]",
						pluginDependency.getPluginId(),
						pluginDependency.getPluginVersion());

				PluginInfo dependPluginInfo = getDependencyPluginInfo(pluginDependency);
				assemble(dependPluginInfo);
				setDependencyService(pluginInfo, dependPluginInfo,
						pluginDependency);

				logger.logMessage(LogLevel.INFO,
						"装载依赖的项[plugin-id:{0},plugin-version:{1}]完成",
						pluginDependency.getPluginId(),
						pluginDependency.getPluginVersion());
			}
		}
		logger.logMessage(LogLevel.INFO,
				"装载插件[id:{0}, version:{1}, type:{2}]依赖的项完成",
				pluginInfo.getId(), pluginInfo.getVersion(),
				pluginInfo.getType());
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void setDependencyService(PluginInfo pluginInfo,
			PluginInfo dependPluginInfo, PluginDependency pluginDependency) {
		String serviceVersion = pluginDependency.getServiceVersion();
		String serviceId = pluginDependency.getServiceId();
		String serviceType = pluginDependency.getServiceType();

		logger.logMessage(
				LogLevel.INFO,
				"设置插件[id:{0}, version:{1}, type:{2}]依赖的服务[plugin-id:{3},plugin-version:{4},service-id={5},service-version={6}]",
				pluginInfo.getId(), pluginInfo.getVersion(),
				pluginInfo.getType(), pluginDependency.getPluginId(),
				pluginDependency.getPluginVersion(),
				pluginDependency.getServiceId(),
				pluginDependency.getServiceVersion());

		try {
			if (serviceVersion == null) {
				if (serviceId == null) {
					Class clazz = Class.forName(serviceType);
					pluginMap.get(pluginInfo).setService(
							pluginMap.get(dependPluginInfo).getService(clazz),
							clazz);

				} else {
					Object object = pluginMap.get(dependPluginInfo).getService(
							serviceId);
					Class clazz = object.getClass();
					pluginMap.get(pluginInfo).setService(object, clazz);
				}
			} else {
				if (serviceId == null) {
					Class clazz = Class.forName(serviceType);
					pluginMap.get(pluginInfo).setService(
							pluginMap.get(dependPluginInfo).getService(clazz,
									serviceVersion), clazz);

				} else {
					Object object = pluginMap.get(dependPluginInfo).getService(
							serviceId, serviceVersion);
					Class clazz = object.getClass();
					pluginMap.get(pluginInfo).setService(object, clazz);
				}
			}

			logger.logMessage(
					LogLevel.INFO,
					"设置插件[id:{0}, version:{1}, type:{2}]依赖的服务[plugin-id:{3},plugin-version:{4},service-id={5},service-version={6}]完成",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), pluginDependency.getPluginId(),
					pluginDependency.getPluginVersion(),
					pluginDependency.getServiceId(),
					pluginDependency.getServiceVersion());
		} catch (Exception e) {
			logger.logMessage(
					LogLevel.INFO,
					"设置插件[id:{0}, version:{1}, type:{2}]依赖的服务[plugin-id:{3},plugin-version:{4},service-id={5},service-version={6}]出错",
					pluginInfo.getId(), pluginInfo.getVersion(),
					pluginInfo.getType(), pluginDependency.getPluginId(),
					pluginDependency.getPluginVersion(),
					pluginDependency.getServiceId(),
					pluginDependency.getServiceVersion());
			throw new PluginException(e.getMessage(), e);
		}
	}

	public void assemble() {
		logger.logMessage(LogLevel.INFO, "开始装载所有插件");
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				assemble(pluginInfo);
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "所有插件已装载");
	}

	public void disassemble(PluginInfo pluginInfo) {

		logger.logMessage(LogLevel.INFO,
				"开始卸载插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());

		int status = status(pluginInfo);

		if (status == PluginManager.STATUS_READY
				|| status == PluginManager.STATUS_PAUSED) {
			stop(pluginInfo);
			status = status(pluginInfo);
		}
		// 只有是停止状态才可以被销毁
		if (status == PluginManager.STATUS_ASSEMBLEED) {
			try {
				setStatus(pluginInfo, PluginManager.STATUS_DESTORYING);
				disassembleDependencyBy(pluginInfo);

				setStatus(pluginInfo, PluginManager.STATUS_INITED);
			} catch (PluginException e) {
				setStatus(pluginInfo, status);
				logger.logMessage(LogLevel.ERROR,
						"插件[id:{0}, version:{1}, type:{2}]卸载时出错，恢复状态为[{3}]",
						pluginInfo.getId(), pluginInfo.getVersion(),
						pluginInfo.getType(), getStatusDescription(status));
				throw e;
			}
		}
		logger.logMessage(LogLevel.INFO,
				"插件[id:{0}, version:{1}, type:{2}]已卸载完成", pluginInfo.getId(),
				pluginInfo.getVersion(), pluginInfo.getType());
	}

	public void disassemble() {
		logger.logMessage(LogLevel.INFO, "开始卸载所有插件");
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				disassemble(pluginInfo);
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "所有插件已卸载");
	}

	public int size() {
		return pluginInfoList.size();
	}

	public List<PluginInfo> getAllPluginInfo() {
		return pluginInfoList;
	}


	public void remove(PluginInfo plugin) {

		logger.logMessage(LogLevel.INFO,
				"开始移除插件[id:{0}, version:{1}, type:{2}]", plugin.getId(),
				plugin.getVersion(), plugin.getType());
		// 删除指定的plugin
		destroy(plugin);
		pluginInfoList.remove(plugin);
		idVersionMap.remove(getKey(plugin));
		pluginMap.remove(plugin);
		pluginStatus.remove(plugin);

		// 如果当前plugin是同类中最高版本(即被存放与idMap)，则将其中map中删除
		if (!idMap.get(plugin.getId()).getVersion().equals(plugin.getVersion())) {

			logger.logMessage(LogLevel.INFO,
					"插件[id:{0}, version:{1}, type:{2}]已移除", plugin.getId(),
					plugin.getVersion(), plugin.getType());

			return;
		}
		logger.logMessage(LogLevel.INFO,
				"插件[id:{0}, version:{1}, type:{2}]是当前插件最高版本", plugin.getId(),
				plugin.getVersion(), plugin.getType());

		idMap.remove(plugin.getId());

		logger.logMessage(LogLevel.INFO, "开始为插件[id:{0}]计算新的最高版本",
				plugin.getId());

		List<PluginInfo> list = new ArrayList<PluginInfo>();
		for (PluginInfo p : pluginInfoList) { // 查询是否有与plugin同id但不同版本的
			if (p.getId().equals(plugin.getId())) {
				list.add(p);
			}
		}
		if (list.size() <= 0) {
			logger.logMessage(LogLevel.INFO, "插件[id:{0}]已无其他版本", plugin.getId());
			logger.logMessage(LogLevel.INFO,
					"插件[id:{0}, version:{1}, type:{2}]已移除", plugin.getId(),
					plugin.getVersion(), plugin.getType());
			return;
		}
		if (list.size() == 1) { // 有且只有一个，则直接将该版本设置为最高版本，放入idMap
			idMap.put(list.get(0).getId(), list.get(0));
			logger.logMessage(LogLevel.INFO, "插件[id:{0}]最新版本为[version:{1}]",
					plugin.getId(), list.get(0).getVersion());
			logger.logMessage(LogLevel.INFO,
					"插件[id:{0}, version:{1}, type:{2}]已移除", plugin.getId(),
					plugin.getVersion(), plugin.getType());
			return;
		}
		PluginInfo newPlugin = list.get(0);
		for (int i = 1; i < list.size(); i++) {
			PluginInfo p1 = list.get(i);
			if (pluginInfoCompare.compare(p1, newPlugin) > 0) {
				newPlugin = p1;
			}
		}
		idMap.put(newPlugin.getId(), newPlugin);
		logger.logMessage(LogLevel.INFO, "插件[id:{0}]最新版本为[version:{1}]",
				plugin.getId(), newPlugin.getVersion());
		logger.logMessage(LogLevel.INFO,
				"插件[id:{0}, version:{1}, type:{2}]已移除", plugin.getId(),
				plugin.getVersion(), plugin.getType());
	}

	public void remove(List<PluginInfo> pluginInfolist) {
		logger.logMessage(LogLevel.INFO, "开始移除指定插件列表");
		for (PluginInfo plugin : pluginInfolist) {
			remove(plugin);
		}
		logger.logMessage(LogLevel.INFO, "移除指定插件列表完成");
	}

	public void removeAll() {
		logger.logMessage(LogLevel.INFO, "开始移除所有插件");

		for (int i = pluginInfoList.size() - 1; i >= 0; i--) {
			try {
				remove(pluginInfoList.get(i));
			} catch (PluginException e) {
				processException(e);
			}

		}
		logger.logMessage(LogLevel.INFO, "移除所有插件完成");
	}

	public void init(int level) {
		logger.logMessage(LogLevel.INFO, "开始初始化插件,初始化等级{0}", level);
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				if (pluginInfo.getLevel() <= level) {
					init(pluginInfo);
				}
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "初始化插件完成,初始化等级{0}", level);
	}

	public void assemble(int level) {
		logger.logMessage(LogLevel.INFO, "开始装载插件,装载等级{0}", level);
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				if (pluginInfo.getLevel() <= level) {
					assemble(pluginInfo);
				}
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "装载插件完成,装载等级{0}", level);
	}

	public void start(int level) {
		logger.logMessage(LogLevel.INFO, "开始启动插件,启动等级{0}", level);
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				if (pluginInfo.getLevel() <= level) {
					start(pluginInfo);
				}
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "启动插件完成,启动等级{0}", level);
	}

	public void pause(int level) {
		logger.logMessage(LogLevel.INFO, "开始暂停插件,暂停等级{0}", level);
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				if (pluginInfo.getLevel() >= level) {
					pause(pluginInfo);
				}
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "暂停插件完成,暂停等级{0}", level);
	}

	public void stop(int level) {
		logger.logMessage(LogLevel.INFO, "开始停止插件,停止等级{0}", level);
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				if (pluginInfo.getLevel() >= level) {
					stop(pluginInfo);
				}
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "停止插件完成,停止等级{0}", level);
	}

	public void disassemble(int level) {
		logger.logMessage(LogLevel.INFO, "开始卸载插件,卸载等级{0}", level);
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				if (pluginInfo.getLevel() >= level) {
					disassemble(pluginInfo);
				}
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "卸载插件完成,卸载等级{0}", level);
	}

	public void destroy(int level) {
		logger.logMessage(LogLevel.INFO, "开始销毁插件,销毁等级{0}", level);
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				if (pluginInfo.getLevel() >= level) {
					destroy(pluginInfo);
				}
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "销毁插件完成,销毁等级{0}", level);
	}

	public String getStatusDescription(int status) {
		switch (status) {
		case STATUS_UNINITIALIZED:
			return "未初始化";
		case STATUS_INITING:
			return "正在初始化";
		case STATUS_INITED:
			return "已初始化";
		case STATUS_ASSEMBLEING:
			return "正在装配";
		case STATUS_ASSEMBLEED:
			return "已装配";
		case STATUS_STARTING:
			return "正在启动";
		case STATUS_READY:
			return "已就绪";
		case STATUS_PAUSING:
			return "正在暂停";
		case STATUS_PAUSED:
			return "已暂停";
		case STATUS_STOPING:
			return "正在停止";
		case STATUS_DISASSEMBLEING:
			return "正在卸载(逆装配)";
		case STATUS_DESTORYING:
			return "正在销毁";
		}
		return "状态不可识别";
	}

	public Context getContext() {
		return context;
	}

	private void addConifg(PluginInfo pluginInfo, XmlNode config) {
		pluginConfigs.put(getKey(pluginInfo), config);
	}

	public XmlNode getConifg(PluginInfo pluginInfo) {
		return pluginConfigs.get(getKey(pluginInfo));
	}

	public String getNodeName() {
		return PLUGINCONFIGNODE;
	}

	public void setConfiguration(XmlNode xmlNode) {
		this.config = xmlNode;
		if (this.config == null) {
			logger.logMessage(LogLevel.WARN, "插件配置信息plugin-configs节点为空");
			return;
		}
		NameFilter<XmlNode> nameFilter = new NameFilter<XmlNode>(this.config);
		List<XmlNode> list = nameFilter
				.findNodeList(PluginXmlConfigNode.PLUGIN_CONFIG);
		for (XmlNode node : list) {
			PluginInfo info = getPluginInfo(
					node.getAttribute(PluginXmlConfigNode.PLUGIN_ID),
					node.getAttribute(PluginXmlConfigNode.PLUGIN_VERSION));
			if (info == null) {
				continue;
			}
			addConifg(info, node);
		}
	}

	public XmlNode getConfiguration() {
		return config;
	}


}
